/**
 *     GSM/GPRS  Siemens TC65.
 * 
 *      -,   
 *  SMS    ,   GPIO.
 * 
 * :  ,   RS-232 (115200, 8, n, 1)
 * (, HyperTerminal, term95  minicom).
 * 
 *  SIM-    PIN- "0000"!!!
 * 
 * Author: "Dmitry" <info@gsmpager.spb.ru>
 */

package tc65Test;

import javax.microedition.midlet.*;

import com.siemens.icm.io.*;

public class TC65Test extends MIDlet{
	ATCommand		atCommand; //       .
	MyATCommandListener atCommandListener; //  -    . 

	MenuSettings	menuSettings;
	Tc65Io			tc65Io;
	Tc65SerialPort	tc65SerialPort;
	SMSSendReceive  smsSendReceive;

	public TC65Test()
	{
		try 
		{
			tc65SerialPort = new Tc65SerialPort();
			tc65SerialPort.serialOut("\n\rStarting...\n\r");
			atCommand = new ATCommand(false);
			
			tc65Io = new Tc65Io(atCommand, tc65SerialPort);
			menuSettings = new MenuSettings(tc65SerialPort, atCommand);
			atCommandListener = new MyATCommandListener(tc65SerialPort);
			smsSendReceive = new SMSSendReceive(atCommand, tc65SerialPort);

			atCommand.addListener(atCommandListener);

			//     
			menuSettings.start();

			serialOut("\n\r\n\rTC65 Java app created.\n\r");
		} 
		catch(Exception e) 
		{
			System.out.println(e);
			notifyDestroyed();
		}
	}

	/**
	 *   : ,  PIN...
	 *  false     -   
	 *  PIN  3.     .
	 */	
	void init()
	{
		try
		{
			atCommand.send("AT&F\r"); //   
			Thread.sleep(1000); //    
			atCommand.send("AT+CPIN=0000\r");
			Thread.sleep(5000); //    
		}
		catch(ATCommandFailedException e)
		{
			serialOut("ERROR: init failed\n\r");
		}
		catch (InterruptedException e) 
		{
			serialOut("ERROR: init interrupted\n\r");
		}
	}

	/**
	 * Wrappers for debug output to serial port
	 * @param str
	 */
	void serialOut(String str)
	{
		tc65SerialPort.serialOut(str.getBytes());
	}
	void serialOut(byte[] ar)
	{
		tc65SerialPort.serialOut(ar);
	}

	/**
	 * startApp()
	 */

	public void startApp() throws MIDletStateChangeException 
	{
		serialOut("\n\rStarting TC65 test app...\n\r");
		//   
		if(tc65Io.initIo())
			serialOut("\n\rI/O initialized.\n\r");
		else
			serialOut("\n\rERROR initialization I/O!\n\r");

		serialOut("\n\rSending PIN \"0000\"(wait 6 sec).\n\r");

		init();
		
		//     -
		tc65Io.initIo();

		smsSendReceive.init();

		char FirstCharOfNumber = menuSettings.Number1.charAt(0);
		if(FirstCharOfNumber != '+' && 
				FirstCharOfNumber != '8')
			serialOut("Telephone number is bad or not set.\n\rPress ENTER to see menu.\n\r\n\r");
		else
			serialOut("TC65 Java app is ready for operation.\n\rPress ENTER to see menu.\n\r\n\r");

		int countLoops = 0;

		while(menuSettings.isAlive()) 
		{
			try 
			{
				Thread.sleep(150);
				countLoops++;

				if(menuSettings.SendTestSMS)
				{
					FirstCharOfNumber = menuSettings.Number1.charAt(0);
					if(FirstCharOfNumber != '+' && 
							FirstCharOfNumber != '8')
						serialOut("Telephone number (" + menuSettings.Number1 + ") is bad or not set, cannot send SMS\n\r");					
					else
						NotifyOnEvent(menuSettings.Number1, 
							"   65.   -. www.gsmpager.ru");
					menuSettings.SendTestSMS = false;
				}
				
				ProcessUnsolicitedEvents();
			} 
			catch(Exception ioe) 
			{
				serialOut("Excp "+ioe.getMessage()+">");
			}
		}

		serialOut("\n\rJava app closed");

		destroyApp(true);
	}

	/**
	 *   ,    ( RING  
	 *    SMS)
	 */
	void ProcessUnsolicitedEvents()
	{
		if(atCommandListener.hasNewEvent())
		{
			if(atCommandListener.strATCommand.indexOf("RING") != -1)
			{
				serialOut("\n\rNew event: incoming ring\n\r");			  
			}
			else if(atCommandListener.strATCommand.indexOf("+CMTI:") != -1)
			{
				if(smsSendReceive.receiveSMS())
					serialOut("\n\rNew incoming SMS from " +
							smsSendReceive.pdu.smsSenderID +
							" at " + smsSendReceive.pdu.smsTime +
							": "+smsSendReceive.pdu.smsText+"\n\r");
			}
			else
				serialOut("\n\rNew event (not processed): " + atCommandListener.strATCommand + "\n\r");			  
		}
	}

	/**
	 * pauseApp()
	 */
	public void pauseApp() {
//		System.out.println("RS232Demo: pauseApp()");
	}

	/**
	 * destroyApp()
	 *
	 * This is important.  It closes the app's RecordStore
	 * @param cond true if this is an unconditional destroy
	 *             false if it is not
	 *             currently ignored and treated as true
	 */
	public void destroyApp(boolean cond) {
		tc65Io.closeIo();
		menuSettings.close();
		notifyDestroyed();
	}

	/**
	 *          -
	 * @param num
	 * @param msg
	 */	  
	private void NotifyOnEvent(String num, String msg)
	{
		//   ,     -
		serialOut("\n\rMessage: " + Translit(msg) + "\n\r");

		//   ,      SMS
		smsSendReceive.sendSMS(num, msg);
	}

	/**
	 *      (   -)
	 * @param text
	 * @return
	 */
	private String Translit(String text)
	{
		StringBuffer sb = new StringBuffer(text.length()+10);
		for(int i=0; i<text.length(); i++)
		{
			char c = text.charAt(i);
			switch(c)
			{
			case '': c = 'A';break; case '': c = 'a';break;
			case '': c = 'B';break; case '': c = 'b';break;
			case '': c = 'V';break; case '': c = 'v';break;
			case '': c = 'G';break; case '': c = 'g';break;
			case '': c = 'D';break; case '': c = 'd';break;
			case '': c = 'E';break; case '': c = 'e';break;
			case '': c = 'Z';break; case '': c = 'z';break;
			case '': c = 'Z';break; case '': c = 'z';break;
			case '': c = 'I';break; case '': c = 'i';break;
			case '': c = 'J';break; case '': c = 'j';break;
			case '': c = 'K';break; case '': c = 'k';break;
			case '': c = 'L';break; case '': c = 'l';break;
			case '': c = 'M';break; case '': c = 'm';break;
			case '': c = 'N';break; case '': c = 'n';break;
			case '': c = 'O';break; case '': c = 'o';break;
			case '': c = 'P';break; case '': c = 'p';break;
			case '': c = 'R';break; case '': c = 'r';break;
			case '': c = 'S';break; case '': c = 's';break;
			case '': c = 'T';break; case '': c = 't';break;
			case '': c = 'U';break; case '': c = 'u';break;
			case '': c = 'F';break; case '': c = 'f';break;
			case '': c = 'H';break; case '': c = 'h';break;
			case '': c = 'C';break; case '': c = 'c';break;
			case '': c = 'H';break; case '': c = 'h';break;
			case '': c = 'S';break; case '': c = 's';break;
			case '': c = 'S';break; case '': c = 's';break;
			case '': c = '\'';break; case '': c = '\'';break;
			case '': c = 'Y';break; case '': c = 'y';break;
			case '': c = '\'';break; case '': c = '\'';break;
			case '': c = 'E';break; case '': c = 'e';break;
			case '': c = 'U';break; case '': c = 'u';break;
			case '': c = 'A';break; case '': c = 'a';break;
			}
			sb.append(c);
		}
		return sb.toString();
	}
}
